/*
 * Proyecto MiCasa, version 1.
 * Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> y
 * Jon Lpez de Dicastillo <darwater@hotmail.com>.
 * Curso 2001-2002
 * Facultad de Ingeniera
 * Universidad de Deusto
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package MiCasa;

import java.awt.Dialog;
import java.awt.Panel;
import java.awt.Label;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Frame;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.AWTEvent;

/**
 * Clase para un dilogo estndar de alerta. Varias veces es necesario
 * mostrarle al usuario un mensaje o pedirle una confirmacin. Esta clase
 * genrica permite hacer eso, construyendo un dilogo modal que contiene
 * un texto y uno o dos botones.
 */
public class DialogoAlerta extends Dialog {
   /** Guarda si el usuario puls en aceptar. */
   boolean aceptar = false;

   /**
    * Ajusta/crea todos los elementos grficos.
    * @return El panel principal de la ventana.
    */
   private Panel IniciarDialogo(String texto, String b1, String b2) throws Exception {
      this.setLayout(new BorderLayout());
      Panel panel = new Panel(new BorderLayout());
      this.add(panel, BorderLayout.CENTER);

      panel.add(new Label(texto), BorderLayout.CENTER);

      Panel botones = new Panel();
      panel.add(botones, BorderLayout.SOUTH);
      
      Button boton;
      boton = new Button(b1);
      boton.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) { BotonAceptar(e); }});
      botones.add(boton);

      if(b2 != null) {
         boton = new Button(b2);
         boton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) { BotonCancelar(e); }});
         botones.add(boton);
      }
      return panel;
   }

   /**
    * Constructor del dilogo. Normalmente no querrs usar esto, sino una de
    * las variantes de
    * {@link Pregunta(Frame frame, String texto, String b1, String b2)} o
    * {@link Pregunta(Frame frame, String texto, String b1)}.
    * @param frame Ventana padre.
    * @param texto El mensaje que mostrar el dilogo.
    * @param b1 Texto del primer botn.
    * @param b2 Texto del segundo botn (puede ser NULL).
    */
   public DialogoAlerta(Frame frame, String texto, String b1, String b2) {
      super(frame, "Mensaje", true);
      enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      try {
         add(IniciarDialogo(texto, b1, b2));
         pack();
      }
      catch(Exception ex) {
         ex.printStackTrace();
      }
   }

   /**
    * Gestiona los mensajes de la ventana. Simplemente recoge el de
    * cierre de ventana para cancelar la operacin.
    */
   protected void processWindowEvent(WindowEvent e) {
      if (e.getID() == WindowEvent.WINDOW_CLOSING)
         dispose();
      super.processWindowEvent(e);
   }

   /**
    * Devuelve si se puls el botn de aceptar.
    */
   public boolean Aceptar() { return aceptar; }

   /**
    * Llamado cuando el usuario pincha en el botn de cancelar.
    */
   private void BotonCancelar(ActionEvent e) { dispose(); }

   /**
    * Llamado cuando el usuario pincha en el botn de aceptar.
    */
   private void BotonAceptar(ActionEvent e) { aceptar = true; dispose(); }

   /**
    * Esto es una especie de constructor de conveniencia, que envuelve al
    * constructor real y devuelve simplemente si el usuario puls aceptar
    * o n.
    */
   public static boolean Pregunta(Frame frame, String texto, String b1, String b2) {
      DialogoAlerta d = new DialogoAlerta(frame, texto, b1, b2);
      d.show();
      if(d.Aceptar()) return true;
      return false;
   }

   /**
    * Esto es una especie de constructor de conveniencia, que envuelve al
    * constructor real y devuelve simplemente si el usuario puls aceptar
    * o n.
    */
   public static boolean Pregunta(Frame frame, String texto, String b1) {
      DialogoAlerta d = new DialogoAlerta(frame, texto, b1, null);
      d.show();
      if(d.Aceptar()) return true;
      return false;
   }
}

