/*
 * Proyecto MiCasa, version 1.
 * Copyright (C) 2002 Grzegorz Adam Hankiewicz <gradha@terra.es> y
 * Jon Lpez de Dicastillo <darwater@hotmail.com>.
 * Curso 2001-2002
 * Facultad de Ingeniera
 * Universidad de Deusto
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package MiCasa;

import java.awt.Panel;
import java.awt.TextField;
import java.awt.Label;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Frame;
import java.awt.List;
import java.awt.Color;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.FileDialog;
import java.util.Vector;
import java.io.IOException;
import java.io.FileWriter;

/**
 * La clase VentanaCliente ser la interfaz con la que el usuario realizar
 * las peticiones al servidor y donde ver los resultados.
 */
public class VentanaCliente extends Frame {
   List listaViviendas = new List();
   ConexionCliente cc = null;

   /**
    * Ajusta/crea todos los elementos grficos.
    */
   private void IniciarVentana() throws Exception {
      this.setBackground(Color.lightGray);
      this.setSize(620, 350);
      this.setTitle("MiCasa");
      this.setLayout(new BorderLayout());
      this.add(listaViviendas, BorderLayout.CENTER);
      Panel panel = new Panel(new GridLayout(4, 1));
      this.add(panel, BorderLayout.EAST);

      Button boton;
      boton = new Button("Listado completo");
      boton.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) { BotonListado(e); }});
      panel.add(boton);

      boton = new Button("Listado con patrn");
      boton.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) { BotonPatron(e); }});
      panel.add(boton);

      boton = new Button("Comprar Vivienda");
      boton.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) { BotonComprar(e); }});
      panel.add(boton);
      
      boton = new Button("Salir");
      boton.addActionListener(new java.awt.event.ActionListener() {
         public void actionPerformed(ActionEvent e) { BotonSalir(e); }});
      panel.add(boton);

      this.addWindowListener(new java.awt.event.WindowAdapter() {
         public void windowClosing(WindowEvent e) { CerrarVentana(e); }});
   }

   /**
    * Constructor de la ventana del cliente.
    */
   public VentanaCliente() {
      try {
         DialogoIniciarConexion dic = new DialogoIniciarConexion(this);
         dic.show();
         if(dic.Aceptar()) {
            cc = new ConexionCliente(dic.Host(), dic.Puerto());

            if(AceptarClaveUsuario()) {
               IniciarVentana();
               RefrescarLista();
            }
            else {
               System.out.println("El usuario cancel la autentificacin.");
               Cerrar();
            }
         }
         else {
            System.out.println("El usuario cancel la conexin.");
            Cerrar();
         }
      }
      catch (Exception e) { // Volcamos el error y lo vemos en una ventana
         e.printStackTrace();
         DialogoError d = new DialogoError(this, e);
//       Descomentar las siguientes lneas si se tiene el JDK 1.4
//         GraphicsEnvironment g = GraphicsEnvironment.getLocalGraphicsEnvironment();
//         Rectangle rect = g.getMaximumWindowBounds();
//         d.setSize(rect.width - rect.x, rect.height - rect.y);
//         d.setLocation(rect.x, rect.y);
         d.show();
         Cerrar();
      }
   }

   /**
    * Pide al usuario un nombre de usuario y contrasea.
    * @return False si se cancel todo.
    */
   private boolean AceptarClaveUsuario() throws Exception {
      DialogoUsuarioClave duc = new DialogoUsuarioClave(this, "Introduzca " +
         "su clave y nombre de usuario, por favor.");
      while(true) {
         duc.show();
         if(!duc.Continuar())
            return false;
         if(cc.Autentificar(duc.Usuario(), duc.Clave()))
            return true;
         duc = new DialogoUsuarioClave(this, "Error, datos no vlidos. " +
            "Por favor, reintente introducir sus datos.");
      }
   }

   /**
    * Mtodo que se encarga de desconectar el cliente, cerrar la
    * ventana y cerrar la aplicacin. Se le llama cuando se pulsa
    * el botn cerrar y cuando se cierra la ventana.
    */
   private void Cerrar() {
      if(cc != null) cc.Desconectar();
      dispose();
      System.exit(0);
   }

   /**
    * Mtodo llamado cuando el usuario cierra la ventana manualmente.
    */
   private void CerrarVentana(WindowEvent e) { Cerrar(); }

   /**
    * Llamado cuando el usuario pincha en el botn de salida.
    */
   private void BotonSalir(ActionEvent e) { Cerrar(); }

   /**
    * Actualiza la lista pidindola de nuevo al servidor.
    */
   private void BotonListado(ActionEvent e) { RefrescarLista(); }

   /**
    * Muestra al usuario un dilogo para recoger un patrn de bsqueda.
    */
   private void BotonPatron(ActionEvent e) {   
      DialogoListadoPatron dlp = new DialogoListadoPatron(this);
      dlp.show();
      if(dlp.Aceptar() == false)
         return;
      if(dlp.Patron() != null)
         RefrescarLista(dlp.Patron());
   }

   /**
    * Actualiza la ventana con la lista de todas las casas.
    */
   private void RefrescarLista() { RefrescarLista(null); }

   /**
    * Obtiene la lista de todas las casas segn el patrn.
    */
   private void RefrescarLista(String patron) {
      listaViviendas.removeAll();
      Vector lista = cc.ListadoViviendas(patron);
      for(int f = 0; f < lista.size(); f++)
         listaViviendas.add((String)lista.get(f));
   }

   /**
    * El usuario quiere comprar la casa seleccionada. Se verifica que
    * se puede comprar y se realiza la compra tras confirmarla.
    */
   private void BotonComprar(ActionEvent e) {
      if(listaViviendas.getSelectedIndex() < 0) {
         DialogoAlerta.Pregunta(this, "Antes selecciona una casa", "Vale");
         return;
      }

      int precio = cc.ComprarVivienda(listaViviendas.getSelectedItem());
      if(precio < 0) {
         DialogoAlerta.Pregunta(this, "La vivienda no existe!", "Vale");
         RefrescarLista();
      }
      else if(precio == 0)
         DialogoAlerta.Pregunta(this, "La casa ya est comprada", "Vale");
      else {
         // Una vez entrado aqu hay que confirmar o rechazar A MUERTE
         if(DialogoAlerta.Pregunta(this, "Desea comprar esa casa por " +
            precio + " euros?", "Si", "No, me lo quiero pensar")) {
            cc.ConfirmarCompra(true);
            RefrescarLista();
         }
         else {
            cc.ConfirmarCompra(false);
         }
      }
   }
}

